@@ -27,6 +27,10 @@ lazy_static! {
     .into_iter()
     .map(|x| x.to_string())
     .collect();
+    static ref VEC_PEPS: Vec<char> = vec![
+        'A', 'L', 'W', 'Q', 'Y', 'E', 'C', 'D', 'F', 'G', 'H', 'I', 'M', 'K', 'P', 'R',
+        'S', 'V', 'N', 'T', '*', '-',
+    ];
 }
 
 fn error_out_program(message: &str) {
@@ -97,11 +101,7 @@ impl AminoAcidTranslator {
         let mut amino_acid_filtered = String::new();
 
         amino_acid_trimmed.char_indices().for_each(|(i, c)| {
-            match !vec![
-                'A', 'L', 'W', 'Q', 'Y', 'E', 'C', 'D', 'F', 'G', 'H', 'I', 'M', 'K', 'P', 'R',
-                'S', 'V', 'N', 'T', '*', '-',
-            ]
-            .contains(&c)
+            match VEC_PEPS.contains(&c)
             {
                 true => {
                     amino_acid_filtered.push('X');
@@ -167,15 +167,18 @@ impl AminoAcidTranslator {
 
                                         match original_triplet.contains('N') {
                                             true => {
-                                                let mut ret = String::new();
-
-                                                for s in LIST_UNKNOWNS.iter() {
-                                                    if &original_triplet == s {
-                                                        ret = s.clone()
-                                                    }
-                                                };
-                                                
-                                                return ret;
+                                                match LIST_UNKNOWNS
+                                                    .iter() 
+                                                    .cloned()
+                                                    .filter(|unk| &original_triplet == unk)
+                                                    .next() {
+                                                        Some(s) => s,
+                                                        None => {
+                                                            self.error_out();
+                                                            return "".to_string();
+                                                        },
+                                                    }                                            
+                            
                                             },
                                             false => {
                                                 taxa_mut.retain(|s| s == &original_triplet);
@@ -217,7 +220,7 @@ pub fn pn2codon(
     gene_table: HashMap<char, Vec<String>>,
     amino_seqs: Vec<(String, String)>,
     nuc_seqs: Vec<(String, String)>,
-) -> Vec<(String, String)> {
+) -> String {
     let mut fstem_mutex = FILE_STEM.lock();
     *fstem_mutex = file_steem;
 
@@ -255,9 +258,11 @@ pub fn pn2codon(
 
             let codon = amino_acid.reverse_translate_and_compare(&gene_table);
 
-            (nt_header.clone(), codon)
+            format!("{}\n{}", aa_header, codon)
         })
-        .collect()
+        .collect::<Vec<String>>()
+        .join("\n")
+
 
    
 }
